@@ -161,16 +161,6 @@ class Int(KatcpType):
             raise ValueError("Integer %d is higher than maximum %d."
                 % (value, self._max))
 
-class GaussianInt(KatcpType):
-    """
-    Complex integer numbers as per NRC-KAT7-6.0-IFCE-002 (KATCP guidelines) A.5
-    """
-
-    name = 'gausian_integer'
-
-    def encode(self, value, major):
-        pass
-        
 
 class Float(KatcpType):
     """The KATCP float type.
