@@ -192,6 +192,7 @@ class DeviceServerBase(object):
         self._thread = None
         self._logger = logger
         self._deferred_queue = Queue.Queue(maxsize=self.MAX_DEFERRED_QUEUE_SIZE)
+        self.send_timeout = 5 # Timeout to catch spinning sends
 
         # sockets and data
         self._data_lock = threading.Lock()
@@ -232,7 +233,11 @@ class DeviceServerBase(object):
         if hasattr(socket, 'TCP_NODELAY'):
             # our message packets are small, don't delay sending them.
             sock.setsockopt(socket.SOL_TCP, socket.TCP_NODELAY, 1)
-        sock.bind(bindaddr)
+        try:
+            sock.bind(bindaddr)
+        except Exception, e:
+            self._logger.exception("Unable to bind to %s" % str(bindaddr))
+            raise
         sock.listen(5)
         return sock
 
@@ -291,15 +296,18 @@ class DeviceServerBase(object):
                         reason, full_line))
                     self.tcp_inform(sock, self._log_msg("error", reason, "root"))
                 else:
-                    client_conn = self._sock_connections[sock]
-                    self.handle_message(client_conn, msg)
+                    try:
+                        client_conn = self._sock_connections[sock]
+                    except KeyError:
+                        self._logger.warn(
+                        'Client disconnected while handling received message: %r'
+                        % sock)
+                    else:
+                        self.handle_message(client_conn, msg)
 
-        self._data_lock.acquire()
-        try:
+        with self._data_lock:
             if sock in self._waiting_chunks:
                 self._waiting_chunks[sock] = waiting_chunk + lines[-1]
-        finally:
-            self._data_lock.release()
 
     def handle_message(self, client_conn, msg):
         """Handle messages of all types from clients.
@@ -446,8 +454,15 @@ class DeviceServerBase(object):
         # do not do anything inside here which could call send_message!
         send_failed = False
         lock.acquire()
+        t0 = time.time()
         try:
             while totalsent < datalen:
+                if time.time()-t0 > self.send_timeout:
+                    self._logger.error(
+                        'server._send_msg() timing out after %fs, sent %d/%d bytes'
+                        % (self.send_timeout, totalsent, datalen) )
+                    send_failed = True
+                    break
                 try:
                     sent = sock.send(data[totalsent:])
                 except socket.error, e:
@@ -470,12 +485,15 @@ class DeviceServerBase(object):
                 client_name = sock.getpeername()
             except socket.error:
                 client_name = "<disconnected client>"
-            msg = "Failed to send message to client %s (%s)" % (client_name, e)
+            msg = "Failed to send message to client %s" % (client_name,)
             self._logger.error(msg)
             # Need to get connection before calling _remove_socket()
-            conn = self._sock_connections[sock]
+            conn = self._sock_connections.get(sock)
             self._remove_socket(sock)
-            self.on_client_disconnect(conn, msg, False)
+            # Don't run on_client_disconnect if another thread has beaten us to
+            # the punch of removing the connection object
+            if conn:
+                self.on_client_disconnect(conn, msg, False)
 
     def inform(self, connection, msg):
         """Send an inform message to a particular client.
@@ -703,9 +721,12 @@ class DeviceServerBase(object):
                 else:
                     # client socket died, remove it
                     # Need to get connection before calling _remove_socket()
-                    conn = self._sock_connections[sock]
+                    conn = self._sock_connections.get(sock)
                     self._remove_socket(sock)
-                    self.on_client_disconnect(conn, "Client socket died", False)
+                    # Don't call on_client_disconnect if the connection has
+                    # already been removed in another thread
+                    if conn:
+                        self.on_client_disconnect(conn, "Client socket died", False)
 
             for sock in readers:
                 if sock is self._sock:
@@ -714,7 +735,13 @@ class DeviceServerBase(object):
                     self.mass_inform(Message.inform("client-connected",
                         "New client connected from %s" % (addr,)))
                     self._add_socket(client)
-                    self.on_client_connect(self._sock_connections[client])
+                    conn = self._sock_connections.get(client)
+                    if client:
+                        self.on_client_connect(conn)
+                    else:
+                        self._logger.warn(
+                            'Client connection for socket %s dissappeared before '
+                            'on_client_connect could be called' % (client,))
                 else:
                     try:
                         chunk = sock.recv(4096)
@@ -727,14 +754,19 @@ class DeviceServerBase(object):
                     else:
                         # no data, assume socket EOF
                         # Need to get connection before calling _remove_socket()
-                        conn = self._sock_connections[sock]
+                        conn = self._sock_connections.get(sock)
                         self._remove_socket(sock)
-                        self.on_client_disconnect(conn, "Socket EOF", False)
+                        # Don't run on_client_disconnect if another thread has
+                        # beaten us to the punch of removing the connection
+                        # object
+                        if conn:
+                            self.on_client_disconnect(conn, "Socket EOF", False)
 
         for sock in list(self._socks):
-            conn = self._sock_connections[sock]
-            self.on_client_disconnect(
-                conn, "Device server shutting down.", True)
+            conn = self._sock_connections.get(sock)
+            if conn:
+                self.on_client_disconnect(
+                    conn, "Device server shutting down.", True)
             self._process_deferred_queue()
             self._remove_socket(sock)
 
@@ -1533,7 +1565,7 @@ class DeviceServer(DeviceServerBase):
         name = msg.arguments[0]
 
         if name not in self._sensors:
-            raise FailReply("Unknown sensor name.")
+            raise FailReply("Unknown sensor name: %s." % name)
 
         sensor = self._sensors[name]
         # The client connection that is not specific to this request context
@@ -1546,19 +1578,19 @@ class DeviceServer(DeviceServerBase):
             params = msg.arguments[2:]
 
             if strategy not in SampleStrategy.SAMPLING_LOOKUP_REV:
-                raise FailReply("Unknown strategy name.")
+                raise FailReply("Unknown strategy name: %s." % strategy)
 
             if not self.PROTOCOL_INFO.strategy_allowed(strategy):
-                raise FailReply("Strategy not allowed for version %d of katcp"
-                                % katcp_version)
+                raise FailReply("Strategy %s not allowed for version %d of katcp"
+                                % (strategy, katcp_version) )
 
             format_inform = (format_inform_v5 if katcp_version >= SEC_TS_KATCP_MAJOR
                              else format_inform_v4)
 
             def inform_callback(sensor_name, timestamp, status, value):
+                """Inform callback for sensor strategy."""
                 cb_msg = format_inform(
                 sensor_name, timestamp, status, value)
-                """Inform callback for sensor strategy."""
                 client.inform(cb_msg)
 
             if katcp_version < SEC_TS_KATCP_MAJOR and strategy == 'period':
